<!doctype html>
<html>
    <head>
        <title>Timetra</title>
        <link href="{{ url_for('static', filename='css/bootstrap.min.css') }}" rel="stylesheet">
        <style type="text/css">
            body {
                padding-top: 50px;
            }
        </style>
    </head>
    <body>
    <div class="container">
        <div class="navbar navbar-fixed-top">
            <div class="navbar-inner">
                <div class="container">
                    <a class="brand" href="/">Timetra</a>

                    <form action="{{ url_for('timetra.search') }}" class="navbar-search pull-left">
                        <input name="q" type="text" class="search-query" placeholder="Search facts">
                    </form>

                    <form action="{{ url_for('timetra.add_fact') }}" method="POST" class="navbar-form pull-right">
                        <div class="input-append">
                            <input name="loose_name" type="text" class="add-on" placeholder="New Activity"><button class="btn btn-success" type="submit">Start!</button>
                        </div>
                    </form>
                </div>
            </div>
        </div>

        <h1>{% block heading %} Timetra {% endblock %}</h1>

        {% with messages = get_flashed_messages(with_categories=true) %}
            {% for category, message in messages %}
                <div class="alert alert-{{ category }}">{{ message|safe }}</div>
            {% endfor %}
        {% endwith %}

        {% block content %}
            <p>(nothing here yet)</p>
        {% endblock %}
    </div>
    </body>
</html>
