{% extends 'base.html' %}

{% from "_helpers.html" import render_tags, render_delta %}
{% from "_formhelpers.html" import render_field %}

{% block heading %} Adding fact {% endblock %}
{% block content %}
    <form method="POST" action="" enctype="multipart/form-data" class="form-horizontal">

        {% for field in form %}
            {{ render_field(field) }}
        {% endfor %}

        <div class="form-actions">
            <input type="submit" value="Add this fact" class="btn-success" />
        </div>
    </form>
{% endblock %}
