{% extends 'base.html' %}

{% from "_helpers.html" import render_tags, render_delta, render_facts %}

{% block heading %} Search results {% endblock %}
{% block content %}
    <p>Found {{ facts|count }} facts.</p>
    {{ render_facts(facts) }}
{% endblock %}
